import simplejson as json

from django import template
from django.core.serializers import serialize
from django.db.models import QuerySet
from django.template.loader import get_template
from django.urls import reverse, resolve
from django.utils.encoding import force_str
from django.utils.functional import Promise
from django.utils.safestring import mark_safe

register = template.Library()


@register.simple_tag
def get_data(row, column):
    return row[column["name"]]


def jsonify(object):
    def date_handler(obj):
        if hasattr(obj, "isoformat"):
            return obj.isoformat()
        elif isinstance(obj, Promise):
            return force_str(obj)

    if isinstance(object, QuerySet):
        return serialize("json", object)

    return mark_safe(json.dumps(object, use_decimal=True, default=date_handler))


register.filter("jsonify", jsonify)


@register.simple_tag
def get_widget_from_url(url_name=None, url=None, **kwargs):
    _url = ""
    if not (url_name or url):
        raise ValueError("url_name or url must be provided")
    if url_name:
        url = reverse(url_name)
    view = resolve(url)
    kwargs["report"] = view.func.view_class
    kwargs["report_url"] = url
    return get_widget(**kwargs)


@register.simple_tag
def get_widget(report, template_name="", url_name="", report_url=None, **kwargs):
    kwargs["report"] = report
    if not report:
        raise ValueError("report argument is empty. Are you sure you're using the correct report name")
    if not (report_url or url_name):
        raise ValueError("report_url or url_name must be provided")

    # if not report.chart_settings:
    kwargs.setdefault("display_chart", bool(report.chart_settings))
    kwargs.setdefault("display_table", True)

    kwargs.setdefault("display_chart_selector", kwargs["display_chart"])
    kwargs.setdefault("display_title", True)

    passed_title = kwargs.get("title", None)
    kwargs["title"] = passed_title or report.get_report_title()
    kwargs["report_url"] = report_url
    if not report_url:
        kwargs["report_url"] = reverse(url_name)

    kwargs.setdefault("extra_params", "")

    template = get_template(template_name or "slick_reporting/widget_template.html")

    return template.render(context=kwargs)



from django.conf import settings
from django.utils.functional import lazy

import datetime



def get_first_of_this_year():
    d = datetime.datetime.today()
    return datetime.datetime(d.year, 1, 1, 0, 0)


def get_end_of_this_year():
    d = datetime.datetime.today()
    return datetime.datetime(d.year + 1, 1, 1, 0, 0)


def get_start_date():
    start_date = getattr(settings, "SLICK_REPORTING_DEFAULT_START_DATE", False)
    return start_date or get_first_of_this_year()


def get_end_date():
    end_date = getattr(settings, "SLICK_REPORTING_DEFAULT_END_DATE", False)
    return end_date or datetime.datetime.today()


SLICK_REPORTING_DEFAULT_START_DATE = lazy(get_start_date, datetime.datetime)()
SLICK_REPORTING_DEFAULT_END_DATE = lazy(get_end_date, datetime.datetime)()

SLICK_REPORTING_FORM_MEDIA_DEFAULT = {
    # "css": {
    #     "all": (
    #         "https://cdn.datatables.net/v/bs4/dt-1.10.20/datatables.min.css",
    #         "https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.9.3/Chart.min.css",
    #     )
    # },
    # "js": (
    #     "https://code.jquery.com/jquery-3.3.1.slim.min.js",
    #     "https://cdn.datatables.net/v/bs4/dt-1.10.20/datatables.min.js",
    #     "https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.9.3/Chart.bundle.min.js",
    #     "https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.9.3/Chart.min.js",
    #     "https://code.highcharts.com/highcharts.js",
    # ),
}

SLICK_REPORTING_FORM_MEDIA = getattr(
    settings, "SLICK_REPORTING_FORM_MEDIA", SLICK_REPORTING_FORM_MEDIA_DEFAULT
)
SLICK_REPORTING_DEFAULT_CHARTS_ENGINE = getattr(
    settings, "SLICK_REPORTING_DEFAULT_CHARTS_ENGINE", "highcharts"
)
